@@ -1,4 +1,4 @@
-from concurrent.futures.thread import ThreadPoolExecutor
+import asyncio
 from http import HTTPStatus
 from typing import List, Dict
 
@@ -44,33 +44,31 @@ class ConsulClient:
     def _execute_bulk_txn(self, keys: List[str], session_id, fn, revert_fn):
         keys = list(set(keys))
         batched_keys = chunk(it=keys, size=self._chunk_size)
-        with ThreadPoolExecutor() as executor:
-            futures = [
-                executor.submit(fn, keys=ks, session_id=session_id)
-                for ks in batched_keys
-            ]
 
-            success, exceptions, failed = [], [], []
-            for i, future in enumerate(futures):
-                try:
-                    result = future.result()
-                except RuntimeError as e:
-                    exceptions.append(str(e))
-                else:
-                    success.append(batched_keys[i]) if result else failed.append(
-                        batched_keys[i]
-                    )
+        loop = asyncio.new_event_loop()
+        asyncio.set_event_loop(loop)
+        tasks = [fn(keys=ks, session_id=session_id) for ks in batched_keys]
+        results = loop.run_until_complete(
+            asyncio.gather(*tasks, return_exceptions=True)
+        )
+        loop.close()
+        success, exceptions, failed = [], [], []
+        for i, r in enumerate(results):
+            if isinstance(r, RuntimeError):
+                exceptions.append(str(r))
+            else:
+                success.append(batched_keys[i]) if r else failed.append(batched_keys[i])
 
-            if exceptions or failed:
-                if success:
-                    r_keys = [ss for s in success for ss in s]
-                    revert_fn(keys=r_keys, session_id=session_id)
-                if exceptions:
-                    raise RuntimeError(f"Consul request failed. {exceptions}")
+        if exceptions or failed:
+            if success:
+                r_keys = [ss for s in success for ss in s]
+                revert_fn(keys=r_keys, session_id=session_id)
+            if exceptions:
+                raise RuntimeError(f"Consul request failed. {exceptions}")
 
-            return len(success) == len(batched_keys)
+        return len(success) == len(batched_keys)
 
-    def _acquire_locks(self, keys, session_id):
+    async def _acquire_locks(self, keys, session_id):
         @retry(
             stop_max_attempt_number=3,
             retry_on_exception=lambda exc: isinstance(exc, RuntimeError),
@@ -86,7 +84,7 @@ class ConsulClient:
         except RetryError as e:
             return False
 
-    def _release_locks(self, keys, session_id):
+    async def _release_locks(self, keys, session_id):
         @retry(
             stop_max_attempt_number=3,
             retry_on_exception=lambda exc: isinstance(exc, RuntimeError),
